package jatva.lang.patterns;

import jatva.lang.ModelEventHelper;
import jatva.lang.ModelObject;
import java.util.Arrays;

/**
 * Describes model events by defining the concrete values of all or
 * some of event attributes.
 *
 * This pattern matches an event if for each attribute which
 * value is defined in this pattern the same attribute of the given
 * event has the same value.
 *
 * @author <a href="mailto:monakhov@ispras.ru">Alexander Monakhov</a>
 */
public class AttributeEventPattern extends ModelEventPattern
{
    private String _name = null;
    private String _prettySignature = null;
    private Object[] _args = null;
    private ModelObject _modelObject;
    private Object _channelId = null;

    public AttributeEventPattern()
    { super(); }

    public AttributeEventPattern( String operationName )
    {
        withName( operationName );
    }

    public AttributeEventPattern setArgsNum( int args_num )
    {
        _args = new Object[ args_num ];
        Arrays.fill( _args, ANY_VALUE );
        return this;
    }

    public AttributeEventPattern withName( String name )
    {
        this._name = name;
        return this;
    }

    public AttributeEventPattern withSignature( String prettySignature )
    {
        this._prettySignature = prettySignature;
        return this;
    }

    public AttributeEventPattern withArgs( Object ... args )
    {
        this._args = args;
        return this;
    }

    public AttributeEventPattern withArg( int i, Object arg )
    {
        _args[ i ] = arg;
        return this;
    }

    public AttributeEventPattern invokedOn( ModelObject modelObject )
    {
        this._modelObject = modelObject;
        return this;
    }

    public AttributeEventPattern inChannel( Object channelId )
    {
        this._channelId = channelId;
        return this;
    }

    /**
     * Indicates that expected event must be registered in the
     * same channel as the event which oracle is being executed now.
     * This method must be invoked in oracles only 'cause there's now
     * current event in scenarios during the frame.
     */
    public AttributeEventPattern inTheSameChannel()
    {
        return inChannel( ModelEventHelper.getCurrentEvent().getChannelId() );
    }

    @Override
    public boolean condition()
    {
        if(    _name != null
            && !(_name.equals( name() ))
          )
        {
            return false;
        }

        if(    _prettySignature != null
            && !(signature().endsWith( this._prettySignature ))
          )
        {
            return false;
        }

        if(    _modelObject != null
            && _modelObject != object()
          )
        {
            return false;
        }

        if( _args != null )
        {
            if( _args.length != args().length )
            {
                return false;
            }
            for( int i = 0; i < _args.length; i ++ )
            {
                if( _args[ i ] == ANY_VALUE )
                    continue;
                if( !(compareObjects( _args[ i ], args()[ i ] )) )
                    return false;
            }
        }

        if(    _channelId != null
            && _channelId != channel()
          )
        {
            return false;
        }

        return true;
    }

    private static final Object ANY_VALUE = new Object();
}
