package jatva.lang.patterns;

import jatva.lang.ModelCall;
import jatva.lang.ModelEvent;
import jatva.lang.ModelEventHelper;
import jatva.lang.ModelObject;
import jatva.lang.ModelReaction;

import java.util.Arrays;

import com.unitesk.utils.predicates.Predicate;

/**
 * Model event pattern is a predicate on an event occurred during the test frame.
 *
 * Rather than implementing {@link Predicate#holdsOn(Object)} method non-abstract
 * ancestors of this class must define an implementation of {@link #condition()} method.
 * During execution of the latter method one can use accessor methods to get various attributes
 * of the event which is currently being checked against this pattern.
 *
 * @author <a href="mailto:monakhov@ispras.ru">Alexander Monakhov</a>
 */
public abstract class ModelEventPattern implements Predicate<ModelEvent>
{
    public ModelEventPattern()
    {
    }

    protected abstract boolean condition();

    public final boolean holdsOn( ModelEvent me )
    {
        return matches( me );
    }

    protected ModelEvent me = null;

    public final boolean matches( ModelEvent me )
    {
        assert this.me == null;

        this.me = me;
        boolean res = condition();
        this.me = null;
        return res;
    }

    // ------------------------------------------------------
    //   ACCESSORS TO EVENT ATTRIBUTES

    protected boolean isReaction()
    {
        return me instanceof ModelReaction;
    }

    protected boolean isStimulus()
    {
        return me instanceof ModelCall;
    }

    protected String name()
    {
        return ModelEventHelper.getOperationName( me );
    }

    protected String signature()
    {
        return ModelEventHelper.getPrettySignature( me );
    }

    protected Object[] args()
    {
        return me.getArguments();
    }

    protected ModelObject object()
    {
        return me.getObject();
    }

    protected Object channel()
    {
        return me.getChannelId();
    }

    protected <T> T arg( int arg_num )
    {
        return ModelEventHelper.<T>getArgument( me, arg_num );
    }

    protected <T> T arg( String arg_name )
    {
        return ModelEventHelper.<T>getArgument( me, arg_name );
    }


    // ------------------------------------------------------
    //   AUXILIARY OPERATIONS
    public boolean compareObjects( Object o1, Object o2 )
    {
        if( o1 == null )
        {
            return o2 == null;
        } else {
            if(    o1.getClass().isArray()
                && o2 != null
                && o2.getClass().isArray()
              )
            {
                return Arrays.deepEquals( (Object[])o1, (Object[])o2 );
            } else {
                return o1.equals( o2 );
            }
        }
    }

}
